package sun.font;

import java.awt.font.GlyphVector;
import sun.java2d.loops.FontInfo;

public final class GlyphList
{
    private static final int MINGRAYLENGTH = 1024;
    private static final int MAXGRAYLENGTH = 8192;
    private static final int DEFAULT_LENGTH = 32;
    int glyphindex;
    int[] metrics;
    byte[] graybits;
    Object strikelist;
    int len;
    int maxLen;
    int maxPosLen;
    int[] glyphData;
    char[] chData;
    long[] images;
    float[] positions;
    float x;
    float y;
    float gposx;
    float gposy;
    boolean usePositions;
    boolean lcdRGBOrder;
    boolean lcdSubPixPos;
    private static GlyphList reusableGL;
    private static boolean inUse;
    
    void ensureCapacity(int n) {
        if (n < 0) {
            n = 0;
        }
        if (this.usePositions && n > this.maxPosLen) {
            this.positions = new float[n * 2 + 2];
            this.maxPosLen = n;
        }
        if (this.maxLen == 0 || n > this.maxLen) {
            this.glyphData = new int[n];
            this.chData = new char[n];
            this.images = new long[n];
            this.maxLen = n;
        }
    }
    
    private GlyphList() {
        this.len = 0;
        this.maxLen = 0;
        this.maxPosLen = 0;
    }
    
    public static GlyphList getInstance() {
        if (GlyphList.inUse) {
            return new GlyphList();
        }
        synchronized (GlyphList.class) {
            if (GlyphList.inUse) {
                return new GlyphList();
            }
            GlyphList.inUse = true;
            return GlyphList.reusableGL;
        }
    }
    
    public boolean setFromString(FontInfo fontInfo, String s, float x, float y) {
        this.x = x;
        this.y = y;
        this.strikelist = fontInfo.fontStrike;
        this.lcdRGBOrder = fontInfo.lcdRGBOrder;
        this.lcdSubPixPos = fontInfo.lcdSubPixPos;
        this.ensureCapacity(this.len = s.length());
        s.getChars(0, this.len, this.chData, 0);
        return this.mapChars(fontInfo, this.len);
    }
    
    public boolean setFromChars(FontInfo fontInfo, char[] array, int n, int n2, float x, float y) {
        this.x = x;
        this.y = y;
        this.strikelist = fontInfo.fontStrike;
        this.lcdRGBOrder = fontInfo.lcdRGBOrder;
        this.lcdSubPixPos = fontInfo.lcdSubPixPos;
        this.len = n2;
        if (n2 < 0) {
            this.len = 0;
        }
        else {
            this.len = n2;
        }
        this.ensureCapacity(this.len);
        System.arraycopy(array, n, this.chData, 0, this.len);
        return this.mapChars(fontInfo, this.len);
    }
    
    private final boolean mapChars(FontInfo fontInfo, int n) {
        if (fontInfo.font2D.getMapper().charsToGlyphsNS(n, this.chData, this.glyphData)) {
            return false;
        }
        fontInfo.fontStrike.getGlyphImagePtrs(this.glyphData, this.images, n);
        this.glyphindex = -1;
        return true;
    }
    
    public void setFromGlyphVector(FontInfo fontInfo, GlyphVector glyphVector, float x, float y) {
        this.x = x;
        this.y = y;
        this.lcdRGBOrder = fontInfo.lcdRGBOrder;
        this.lcdSubPixPos = fontInfo.lcdSubPixPos;
        StandardGlyphVector standardGV = StandardGlyphVector.getStandardGV(glyphVector, fontInfo);
        this.usePositions = standardGV.needsPositions(fontInfo.devTx);
        this.ensureCapacity(this.len = standardGV.getNumGlyphs());
        this.strikelist = standardGV.setupGlyphImages(this.images, this.usePositions ? this.positions : null, fontInfo.devTx);
        this.glyphindex = -1;
    }
    
    public int[] getBounds() {
        if (this.glyphindex >= 0) {
            throw new InternalError("calling getBounds after setGlyphIndex");
        }
        if (this.metrics == null) {
            this.metrics = new int[5];
        }
        this.gposx = this.x + 0.5f;
        this.gposy = this.y + 0.5f;
        this.fillBounds(this.metrics);
        return this.metrics;
    }
    
    public void setGlyphIndex(int glyphindex) {
        this.glyphindex = glyphindex;
        if (this.images[glyphindex] == 0L) {
            this.metrics[0] = (int)this.gposx;
            this.metrics[1] = (int)this.gposy;
            this.metrics[2] = 0;
            this.metrics[3] = 0;
            this.metrics[4] = 0;
            return;
        }
        float float1 = StrikeCache.unsafe.getFloat(this.images[glyphindex] + (long)StrikeCache.topLeftXOffset);
        float float2 = StrikeCache.unsafe.getFloat(this.images[glyphindex] + (long)StrikeCache.topLeftYOffset);
        if (this.usePositions) {
            this.metrics[0] = (int)Math.floor((double)(this.positions[glyphindex << 1] + this.gposx + float1));
            this.metrics[1] = (int)Math.floor((double)(this.positions[(glyphindex << 1) + 1] + this.gposy + float2));
        }
        else {
            this.metrics[0] = (int)Math.floor((double)(this.gposx + float1));
            this.metrics[1] = (int)Math.floor((double)(this.gposy + float2));
            this.gposx += StrikeCache.unsafe.getFloat(this.images[glyphindex] + (long)StrikeCache.xAdvanceOffset);
            this.gposy += StrikeCache.unsafe.getFloat(this.images[glyphindex] + (long)StrikeCache.yAdvanceOffset);
        }
        this.metrics[2] = StrikeCache.unsafe.getChar(this.images[glyphindex] + (long)StrikeCache.widthOffset);
        this.metrics[3] = StrikeCache.unsafe.getChar(this.images[glyphindex] + (long)StrikeCache.heightOffset);
        this.metrics[4] = StrikeCache.unsafe.getChar(this.images[glyphindex] + (long)StrikeCache.rowBytesOffset);
        return;
    }
    
    public int[] getMetrics() {
        return this.metrics;
    }
    
    public byte[] getGrayBits() {
        int a = this.metrics[4] * this.metrics[3];
        if (this.graybits == null) {
            this.graybits = new byte[Math.max(a, 1024)];
        }
        else if (a > this.graybits.length) {
            this.graybits = new byte[a];
        }
        if (this.images[this.glyphindex] == 0L) {
            return this.graybits;
        }
        long address = StrikeCache.unsafe.getAddress(this.images[this.glyphindex] + (long)StrikeCache.pixelDataOffset);
        if (address == 0L) {
            return this.graybits;
        }
        for (int i = 0; i < a; ++i) {
            this.graybits[i] = StrikeCache.unsafe.getByte(address + (long)i);
        }
        return this.graybits;
    }
    
    public long[] getImages() {
        return this.images;
    }
    
    public boolean usePositions() {
        return this.usePositions;
    }
    
    public float[] getPositions() {
        return this.positions;
    }
    
    public float getX() {
        return this.x;
    }
    
    public float getY() {
        return this.y;
    }
    
    public Object getStrike() {
        return this.strikelist;
    }
    
    public boolean isSubPixPos() {
        return this.lcdSubPixPos;
    }
    
    public boolean isRGBOrder() {
        return this.lcdRGBOrder;
    }
    
    public void dispose() {
        if (this == GlyphList.reusableGL) {
            if (this.graybits != null && this.graybits.length > 8192) {
                this.graybits = null;
            }
            this.usePositions = false;
            this.strikelist = null;
            GlyphList.inUse = false;
        }
    }
    
    public int getNumGlyphs() {
        return this.len;
    }
    
    private void fillBounds(int[] array) {
        int topLeftXOffset = StrikeCache.topLeftXOffset;
        int topLeftYOffset = StrikeCache.topLeftYOffset;
        int widthOffset = StrikeCache.widthOffset;
        int heightOffset = StrikeCache.heightOffset;
        int xAdvanceOffset = StrikeCache.xAdvanceOffset;
        int yAdvanceOffset = StrikeCache.yAdvanceOffset;
        if (this.len == 0) {
            int n12 = 0;
            int n13 = 1;
            int n14 = 2;
            int n15 = 3;
            int n16 = 0;
            array[n14] = (array[n15] = n16);
            array[n12] = (array[n13] = n16);
            return;
        }
        float n2;
        float n = n2 = Float.POSITIVE_INFINITY;
        float n4;
        float n3 = n4 = Float.NEGATIVE_INFINITY;
        int n5 = 0;
        float n6 = this.x + 0.5f;
        float n7 = this.y + 0.5f;
        for (int i = 0; i < this.len; ++i) {
            if (this.images[i] != 0L) {
                float float1 = StrikeCache.unsafe.getFloat(this.images[i] + (long)topLeftXOffset);
                float float2 = StrikeCache.unsafe.getFloat(this.images[i] + (long)topLeftYOffset);
                int char1 = StrikeCache.unsafe.getChar(this.images[i] + (long)widthOffset);
                int char2 = StrikeCache.unsafe.getChar(this.images[i] + (long)heightOffset);
                float n8;
                float n9;
                if (this.usePositions) {
                    n8 = this.positions[n5++] + float1 + n6;
                    n9 = this.positions[n5++] + float2 + n7;
                }
                else {
                    n8 = n6 + float1;
                    n9 = n7 + float2;
                    n6 += StrikeCache.unsafe.getFloat(this.images[i] + (long)xAdvanceOffset);
                    n7 += StrikeCache.unsafe.getFloat(this.images[i] + (long)yAdvanceOffset);
                }
                float n10 = n8 + (float)char1;
                float n11 = n9 + (float)char2;
                if (n2 > n8) {
                    n2 = n8;
                }
                if (n > n9) {
                    n = n9;
                }
                if (n4 < n10) {
                    n4 = n10;
                }
                if (n3 < n11) {
                    n3 = n11;
                }
            }
        }
        array[0] = (int)Math.floor((double)n2);
        array[1] = (int)Math.floor((double)n);
        array[2] = (int)Math.floor((double)n4);
        array[3] = (int)Math.floor((double)n3);
        return;
    }
    
    static {
        GlyphList.reusableGL = new GlyphList();
    }
}
